@@ -26,22 +26,6 @@ from ..utils.kernels import kernel_names, kernel_radius
 from ..utils.math import norm
 
 
-class _SinkUtility:
-    def __init__(self, fn):
-        self.fn = fn
-
-    def __getitem__(self, inp):
-        return self.fn(inp, sinks=True)
-
-    def __repr__(self):
-        """Dunder repr method."""
-        return self.__str__()
-
-    def __str__(self):
-        """Dunder str method."""
-        return f'<plonk.snap sinks>'
-
-
 class Snap:
     """Smoothed particle hydrodynamics Snap object.
 
@@ -295,16 +279,16 @@ class Snap:
         else:
             loaded = self.loaded_arrays()
             registered = list(sorted(self._array_registry.keys()))
-            if all:
-                for arr in self._vector_arrays:
-                    if arr in registered:
-                        registered += [f'{arr}_x', f'{arr}_y', f'{arr}_z', f'{arr}_mag']
-                for arr in self._dust_arrays:
-                    if arr in registered:
-                        registered += [
-                            f'{arr}_{n:03}' for n in range(1, self.num_dust_species + 1)
-                        ]
-                        registered.append(f'{arr}_tot')
+        if all:
+            for arr in self._vector_arrays:
+                if arr in registered:
+                    registered += [f'{arr}_x', f'{arr}_y', f'{arr}_z', f'{arr}_mag']
+            for arr in self._dust_arrays:
+                if arr in registered:
+                    registered += [
+                        f'{arr}_{n:03}' for n in range(1, self.num_dust_species + 1)
+                    ]
+                    registered.append(f'{arr}_tot')
 
         if aliases:
             extra = tuple(
@@ -337,20 +321,6 @@ class Snap:
         """
         return self._available_arrays(sinks=False, all=all, aliases=aliases)
 
-    def available_sink_arrays(self, aliases: bool = False):
-        """Return a tuple of available sink arrays.
-
-        Parameters
-        ----------
-        aliases
-            If True, return array aliases. Default is False.
-
-        Returns
-        -------
-        A tuple of names of available arrays.
-        """
-        return self._available_arrays(sinks=True, aliases=aliases)
-
     @property
     def properties(self):
         """Snap properties."""
@@ -359,7 +329,7 @@ class Snap:
     @property
     def sinks(self):
         """Sink particle arrays."""
-        return _SinkUtility(self._getitem)
+        return _Sinks(self)
 
     @property
     def num_particles(self):
@@ -374,17 +344,19 @@ class Snap:
         if self._num_particles_of_type == -1:
             int_to_name = {idx: name for name, idx in self.particle_type.items()}
             d = {}
-            for idx, num in enumerate(np.bincount(self['type'])):
+            for idx, num in enumerate(np.bincount(self['type'].magnitude)):
                 if num > 0:
                     if idx == self.particle_type['dust']:
                         # Dust particle sub-type skips zero: 1, 2, 3 ...
                         d['dust'] = list(
-                            np.bincount(self[self['type'] == idx]['sub_type'])[1:]
+                            np.bincount(
+                                self[self['type'] == idx]['sub_type'].magnitude
+                            )[1:]
                         )
                     elif idx == self.particle_type['boundary']:
                         # Boundary particle sub-type: 0 (gas), 1, 2, 3... (dust)
                         d['boundary'] = list(
-                            np.bincount(self[self['type'] == idx]['sub_type'])
+                            np.bincount(self[self['type'] == idx]['sub_type'.magnitude])
                         )
                     else:
                         d[int_to_name[idx]] = num
@@ -419,17 +391,17 @@ class Snap:
             self._arrays = {}
         self._cache_arrays = value
 
-    def unset(self, rotation: bool = False, translation: bool = False):
-        """Unset.
+    def unset(self, rotation: bool = True, translation: bool = True):
+        """Unset transformations.
 
-        Unset some transformations on the Snap data.
+        Unset rotation and translations transformations on the Snap.
 
         Parameters
         ----------
         rotation
-            Set to True to unset rotation. Default is False.
+            Set to True to unset rotation. Default is True.
         translation
-            Set to True to unset translation. Default is False.
+            Set to True to unset translation. Default is True.
         """
         if any((rotation, translation)):
             for arr in self.loaded_arrays():
@@ -446,15 +418,26 @@ class Snap:
 
         return self
 
-    def rotate(self, rotation: Union[ndarray, Rotation]) -> Snap:
+    def rotate(
+        self,
+        rotation: Rotation = None,
+        axis: Union[ndarray, List, Tuple[float, float, float]] = None,
+        angle: float = None,
+    ) -> Snap:
         """Rotate snapshot.
 
+        The rotation can be defined by a scipy Rotation object, or a
+        combination of a vector, specifying a rotation axis, and an
+        angle, specifying the rotation in radians.
+
         Parameters
         ----------
         rotation
-            The rotation as a scipy.spatial.transform.Rotation object
-            or ndarray that can be converted to a Rotation object via
-            Rotation.from_rotvec.
+            The rotation as a scipy.spatial.transform.Rotation object.
+        axis
+            An array specifying a rotation axis, like (x, y, z).
+        angle
+            A float specifying the rotation in radians.
 
         Returns
         -------
@@ -466,20 +449,26 @@ class Snap:
         --------
         Rotate a Snap by π/3 around [1, 1, 0].
 
-        >>> rot = np.array([1, 1, 0])
-        >>> rot = rot * np.pi / 3 * np.linalg.norm(rot)
-        >>> snap.rotate(rot)
+        >>> axis = (1, 1, 0)
+        >>> angle = np.pi / 3
+        >>> snap.rotate(axis=axis, angle=angle)
         """
         logger.debug(f'Rotating snapshot: {self.file_path.name}')
-        if isinstance(rotation, (list, tuple, ndarray)):
-            rotation = Rotation.from_rotvec(rotation)
+        if rotation is not None:
+            if axis is not None or angle is not None:
+                logger.warning('ignoring axis and angle as rotation is passed in')
+            _rotation = rotation
+        else:
+            _rotation = axis / norm(axis) * angle
+        if isinstance(_rotation, (list, tuple, ndarray)):
+            _rotation = Rotation.from_rotvec(_rotation)
         for arr in self._vector_arrays:
             if arr in self.loaded_arrays():
                 array_m, array_u = self._arrays[arr].magnitude, self._arrays[arr].units
-                self._arrays[arr] = rotation.apply(array_m) * array_u
+                self._arrays[arr] = _rotation.apply(array_m) * array_u
             if arr in self.loaded_arrays(sinks=True):
                 array_m, array_u = self._sinks[arr].magnitude, self._sinks[arr].units
-                self._sinks[arr] = rotation.apply(array_m) * array_u
+                self._sinks[arr] = _rotation.apply(array_m) * array_u
         for arr in self._vector_component_arrays:
             if arr in self.loaded_arrays():
                 del self._arrays[arr]
@@ -487,9 +476,9 @@ class Snap:
                 del self._sinks[arr]
 
         if self.rotation is None:
-            self.rotation = rotation
+            self.rotation = _rotation
         else:
-            rot = rotation * self.rotation
+            rot = _rotation * self.rotation
             self.rotation = rot
 
         return self
@@ -1089,7 +1078,7 @@ class Snap:
 
     def _ipython_key_completions_(self):
         """Tab completion for IPython __getitem__ method."""
-        return self.available_arrays()
+        return self.available_arrays(all=True)
 
     def __len__(self):
         """Length as number of particles."""
@@ -1171,6 +1160,57 @@ class SubSnap(Snap):
 SnapLike = Union[Snap, SubSnap]
 
 
+class _Sinks:
+    def __init__(self, base: Snap):
+        self.base = base
+        self._getitem = base._getitem_from_str
+
+    def available_arrays(self, all: bool = False, aliases: bool = False):
+        """Return a tuple of available sink arrays.
+
+        Parameters
+        ----------
+        all
+            TODO. Default is False
+        aliases
+            If True, return array aliases. Default is False.
+
+        Returns
+        -------
+        A tuple of names of available arrays.
+        """
+        return self.base._available_arrays(sinks=True, all=all, aliases=aliases)
+
+    def __setitem__(self, name: str, item: Quantity):
+        if not isinstance(item, Quantity):
+            raise ValueError('"item" must be an array with units, i.e. a Pint Quantity')
+        if item.shape[0] != len(self):
+            raise ValueError('Length of array does not match particle number')
+        self.base._sinks[name] = item
+
+    def __getitem__(self, inp):
+        if not isinstance(inp, str):
+            raise ValueError('Must pass in string to __getitem__')
+        return self._getitem(inp, sinks=True)
+
+    def __len__(self):
+        return len(self.base._sink_registry['mass'](self.base))
+
+    def __repr__(self):
+        """Dunder repr method."""
+        return self.__str__()
+
+    def __str__(self):
+        """Dunder str method."""
+        return f'<plonk.Snap.Sinks "{self.base.file_path.name}">'
+
+    def _ipython_key_completions_(self):
+        """Tab completion for IPython __getitem__ method."""
+        return self.available_arrays(all=True)
+
+    plot = visualize.particle_plot
+
+
 def _str_is_int(string):
     try:
         int(string)
