@@ -5,7 +5,7 @@ simulations using kernel density estimation based interpolation.
 """
 
 from .animation import animation, animation_particles, animation_profiles
-from .functions import get_extent_from_percentile, str_to_units
+from .functions import get_extent_from_percentile
 from .interpolation import interpolate
 from .multi import plot_snaps
 from .visualization import particle_plot, plot
@@ -19,5 +19,4 @@ __all__ = [
     'particle_plot',
     'plot',
     'plot_snaps',
-    'str_to_units',
 ]
