@@ -11,7 +11,9 @@ from typing import TYPE_CHECKING, Tuple
 import numpy as np
 from numpy import ndarray
 
+from .._logging import logger
 from ..snap.utils import get_array_in_code_units
+from ..utils.geometry import distance_from_plane
 from .splash import interpolate_cross_section, interpolate_projection
 
 if TYPE_CHECKING:
@@ -27,6 +29,7 @@ def interpolate(
     x: str = 'x',
     y: str = 'y',
     interp: 'str',
+    slice_normal: Tuple[float, float, float] = None,
     z_slice: float = None,
     extent: Extent,
     **kwargs,
@@ -51,6 +54,9 @@ def interpolate(
         - 'projection' : 2d interpolation via projection to xy-plane
         - 'cross_section' : 3d interpolation via cross-section in
           z-direction
+    slice_normal
+        The normal vector to the plane in which to take the
+        cross-section slice as an array (x, y, z).
     z_slice
         The z-coordinate value of the cross-section slice. Default
         is 0.0.
@@ -83,23 +89,28 @@ def interpolate(
     m = get_array_in_code_units(snap, 'mass')
 
     if interp == 'projection':
-        cross_section = None
+        dist_from_slice = None
+        if slice_normal is not None:
+            logger.warning('ignoring slice_normal for projection')
+        if z_slice is not None:
+            logger.warning('ignoring z_slice for projection')
     elif interp == 'cross_section':
         if z_slice is None:
             z_slice = 0.0
-        cross_section = z_slice
+        if slice_normal is None:
+            slice_normal = np.array([0, 0, 1])
+        dist_from_slice = distance_from_plane(x, y, z, slice_normal, z_slice)
 
     if _quantity.ndim == 1:
         interpolated_data = scalar_interpolation(
             quantity=_quantity,
             x_coordinate=x,
             y_coordinate=y,
-            z_coordinate=z,
+            dist_from_slice=dist_from_slice,
             extent=extent,
             smoothing_length=h,
             particle_mass=m,
             hfact=snap.properties['smoothing_length_factor'],
-            cross_section=cross_section,
             **kwargs,
         )
 
@@ -109,12 +120,11 @@ def interpolate(
             quantity_y=_quantity[:, 1],
             x_coordinate=x,
             y_coordinate=y,
-            z_coordinate=z,
+            dist_from_slice=dist_from_slice,
             extent=extent,
             smoothing_length=h,
             particle_mass=m,
             hfact=snap.properties['smoothing_length_factor'],
-            cross_section=cross_section,
             **kwargs,
         )
 
@@ -129,13 +139,12 @@ def scalar_interpolation(
     quantity: ndarray,
     x_coordinate: ndarray,
     y_coordinate: ndarray,
-    z_coordinate: ndarray = None,
+    dist_from_slice: ndarray = None,
     extent: Extent,
     smoothing_length: ndarray,
     particle_mass: ndarray,
     hfact: float,
     number_of_pixels: Tuple[float, float] = (512, 512),
-    cross_section: float = None,
     density_weighted: bool = None,
 ) -> ndarray:
     """Interpolate scalar quantity to a pixel grid.
@@ -148,9 +157,9 @@ def scalar_interpolation(
         Particle coordinate for x-axis in interpolation.
     y_coordinate
         Particle coordinate for y-axis in interpolation.
-    z_coordinate
-        Particle coordinate for z-axis. Only required for cross section
-        interpolation.
+    dist_from_slice
+        The distance from the cross section slice. Only required for
+        cross section interpolation.
     extent
         The range in the x- and y-direction as (xmin, xmax, ymin, ymax).
     smoothing_length
@@ -178,13 +187,12 @@ def scalar_interpolation(
         quantity=quantity,
         x_coordinate=x_coordinate,
         y_coordinate=y_coordinate,
-        z_coordinate=z_coordinate,
+        dist_from_slice=dist_from_slice,
         extent=extent,
         smoothing_length=smoothing_length,
         particle_mass=particle_mass,
         hfact=hfact,
         number_of_pixels=number_of_pixels,
-        cross_section=cross_section,
         density_weighted=density_weighted,
     )
 
@@ -195,13 +203,12 @@ def vector_interpolation(
     quantity_y: ndarray,
     x_coordinate: ndarray,
     y_coordinate: ndarray,
-    z_coordinate: ndarray = None,
+    dist_from_slice: ndarray = None,
     extent: Extent,
     smoothing_length: ndarray,
     particle_mass: ndarray,
     hfact: float,
     number_of_pixels: Tuple[float, float] = (512, 512),
-    cross_section: float = None,
     density_weighted: bool = None,
 ) -> ndarray:
     """Interpolate scalar quantity to a pixel grid.
@@ -216,9 +223,9 @@ def vector_interpolation(
         Particle coordinate for x-axis in interpolation.
     y_coordinate
         Particle coordinate for y-axis in interpolation.
-    z_coordinate
-        Particle coordinate for z-axis. Only required for cross section
-        interpolation.
+    dist_from_slice
+        The distance from the cross section slice. Only required for
+        cross section interpolation.
     extent
         The range in the x- and y-direction as (xmin, xmax, ymin, ymax).
     smoothing_length
@@ -246,26 +253,24 @@ def vector_interpolation(
         quantity=quantity_x,
         x_coordinate=x_coordinate,
         y_coordinate=y_coordinate,
-        z_coordinate=z_coordinate,
+        dist_from_slice=dist_from_slice,
         extent=extent,
         smoothing_length=smoothing_length,
         particle_mass=particle_mass,
         hfact=hfact,
         number_of_pixels=number_of_pixels,
-        cross_section=cross_section,
         density_weighted=density_weighted,
     )
     vecsmoothy = _interpolate(
         quantity=quantity_y,
         x_coordinate=x_coordinate,
         y_coordinate=y_coordinate,
-        z_coordinate=z_coordinate,
+        dist_from_slice=dist_from_slice,
         extent=extent,
         smoothing_length=smoothing_length,
         particle_mass=particle_mass,
         hfact=hfact,
         number_of_pixels=number_of_pixels,
-        cross_section=cross_section,
         density_weighted=density_weighted,
     )
     return np.stack((np.array(vecsmoothx), np.array(vecsmoothy)))
@@ -276,22 +281,18 @@ def _interpolate(
     quantity: ndarray,
     x_coordinate: ndarray,
     y_coordinate: ndarray,
-    z_coordinate: ndarray = None,
+    dist_from_slice: ndarray = None,
     extent: Extent,
     smoothing_length: ndarray,
     particle_mass: ndarray,
     hfact: float,
     number_of_pixels: Tuple[float, float],
-    cross_section: float = None,
     density_weighted: bool = None,
 ) -> ndarray:
-    if cross_section is None:
+    if dist_from_slice is None:
         do_cross_section = False
     else:
         do_cross_section = True
-        zslice = cross_section
-        if z_coordinate is None:
-            raise ValueError('Cross section interpolation requires z_coordinate')
     normalise = False
     if density_weighted is None:
         density_weighted = False
@@ -314,7 +315,7 @@ def _interpolate(
         interpolated_data = interpolate_cross_section(
             x=x_coordinate,
             y=y_coordinate,
-            z=z_coordinate,
+            dslice=dist_from_slice,
             hh=smoothing_length,
             weight=weight,
             dat=quantity,
@@ -322,7 +323,6 @@ def _interpolate(
             npart=npart,
             xmin=xmin,
             ymin=ymin,
-            zslice=zslice,
             npixx=npixx,
             npixy=npixy,
             pixwidthx=pixwidthx,
